package com.fjhx.account.service.account.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fjhx.account.entity.account.po.AccountManagement;
import com.fjhx.account.entity.account.po.AccountRemainder;
import com.fjhx.account.mapper.account.AccountManagementMapper;
import com.fjhx.account.service.account.AccountManagementService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.account.service.account.AccountRemainderService;
import com.ruoyi.common.utils.StringUtils;
import org.apache.poi.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.account.entity.account.vo.AccountManagementVo;
import com.fjhx.account.entity.account.dto.AccountManagementSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.account.entity.account.dto.AccountManagementDto;
import cn.hutool.core.bean.BeanUtil;

import java.util.List;


/**
 * <p>
 * 账户-管理表 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-06
 */
@Service
public class AccountManagementServiceImpl extends ServiceImpl<AccountManagementMapper, AccountManagement> implements AccountManagementService {
    @Autowired
    private AccountRemainderService accountRemainderService;

    /**
     * 账户-管理表分页
     */
    @Override
    public Page<AccountManagementVo> getPage(AccountManagementSelectDto dto) {
        LambdaQueryWrapper<AccountManagement> wrapper = Wrappers.<AccountManagement>lambdaQuery();
        //根据关键字查询（查询的字段有账户别名、账户名称、账号）
        if (StringUtils.isNotEmpty(dto.getName())){
            wrapper.like(AccountManagement::getName,dto.getName());
            wrapper.like(AccountManagement::getAlias,dto.getName());
            wrapper.like(AccountManagement::getAccountOpening,dto.getName());
        }
        //根据开户银行查询
        wrapper.eq(ObjectUtil.isNotEmpty(dto.getOpeningBank()),AccountManagement::getOpeningBank,dto.getOpeningBank());
        Page<AccountManagementVo> page = baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    /**
     * 账户-管理表明细
     */
    @Override
    public AccountManagementDto detail(Long id) {
        //查询账户管理表的明细
        AccountManagement AccountManagement = this.getById(id);
        AccountManagementDto result = BeanUtil.toBean(AccountManagement, AccountManagementDto.class);
        //查询账户余额表的信息
        List<AccountRemainder> accountRemainderList = accountRemainderService.list(Wrappers.<AccountRemainder>lambdaQuery()
                .eq(AccountRemainder::getAccountManagementId, id));
        result.setAccountRemainderList(accountRemainderList);
        return result;
    }

    /**
     * 账户-管理表新增
     */
    @Override
    public void add(AccountManagementDto accountManagementDto) {
        //添加账户管理表的信息
        this.save(accountManagementDto);
        saveAccountRemainder(accountManagementDto);
    }

    /**
     * 账户-管理表编辑
     */
    @Override
    public void edit(AccountManagementDto accountManagementDto) {
        this.updateById(accountManagementDto);
        //删除账户-余额表的信息
        accountRemainderService.remove(Wrappers.<AccountRemainder>lambdaQuery()
                .eq(AccountRemainder::getAccountManagementId,accountManagementDto.getId()));
        //添加账户余额表的信息
        saveAccountRemainder(accountManagementDto);
    }

    /**
     * 账户-管理表删除
     */
    @Override
    public void delete(Long id) {
        //删除账户-管理表的信息
        this.removeById(id);
        //删除账户-余额表的信息
        accountRemainderService.remove(Wrappers.<AccountRemainder>lambdaQuery()
                .eq(AccountRemainder::getAccountManagementId,id));
    }


    /**
     * 添加账户-余额表的信息
     */
    private void saveAccountRemainder(AccountManagementDto accountManagementDto){
        List<AccountRemainder> accountRemainderList = accountManagementDto.getAccountRemainderList();
        accountRemainderList.forEach(accountRemainder -> {
            accountRemainder.setAccountManagementId(accountManagementDto.getId());
        });
        accountRemainderService.saveBatch(accountRemainderList);
    }
}
