package com.fjhx.supply.service.supplier.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.area.utils.AreaUtil;
import com.fjhx.supply.entity.supplier.dto.SupplierInfoDto;
import com.fjhx.supply.entity.supplier.dto.SupplierInfoSelectDto;
import com.fjhx.supply.entity.supplier.po.SupplierInfo;
import com.fjhx.supply.entity.supplier.vo.SupplierInfoVo;
import com.fjhx.supply.mapper.supplier.SupplierInfoMapper;
import com.fjhx.supply.service.supplier.SupplierInfoService;
import com.fjhx.supply.utils.CodeEnum;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * <p>
 * 供应商 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-17
 */
@Service
public class SupplierInfoServiceImpl extends ServiceImpl<SupplierInfoMapper, SupplierInfo> implements SupplierInfoService {

    @Override
    public Page<SupplierInfoVo> getPage(SupplierInfoSelectDto dto) {
        IWrapper<SupplierInfo> wrapper = getWrapper();
        wrapper.orderByDesc("si", SupplierInfo::getId);
        wrapper.eq("si", SupplierInfo::getType, dto.getType());
        wrapper.keyword(dto,
                new SqlField("si", SupplierInfo::getName),
                new SqlField("si", SupplierInfo::getCode)
        );
        Page<SupplierInfoVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);

        List<SupplierInfoVo> records = page.getRecords();
        if (records.size() == 0) {
            return page;
        }
        AreaUtil.setAreaName(records);
        return page;
    }

    @Override
    public SupplierInfoVo detail(Long id) {
        SupplierInfo SupplierInfo = this.getById(id);
        SupplierInfoVo result = BeanUtil.toBean(SupplierInfo, SupplierInfoVo.class);
        AreaUtil.setAreaName(result);
        return result;
    }

    @Override
    public void add(SupplierInfoDto supplierInfoDto) {
        supplierInfoDto.setCode(CodeEnum.SUPPLIER_INFO.getCode());
        this.save(supplierInfoDto);
    }

    @Override
    public void edit(SupplierInfoDto supplierInfoDto) {
        supplierInfoDto.setCode(null);
        this.updateById(supplierInfoDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
